from collections import OrderedDict

from prometheus_client.core import GaugeMetricFamily
import requests
import xmltodict


class ChillerCollector(object):
    """Collector Class for prometheus client"""

    def __init__(self, app, chillers, metrics):
        """initialize collector"""
        self._app = app
        self._chillers = chillers
        self._metrics = metrics

    def collect(self):
        """callback for prometheus - request data from chiller"""
        metrics = {}

        for instance, (url, timeout) in self._chillers.items():
            labels = {'instance': instance}
            labels = OrderedDict(sorted(labels.items(), key=lambda t: t[0]))

            try:
                with requests.get(url, timeout=timeout) as response:

                    mkey = '{}_http_status'.format(self._app)
                    mhelp = 'Chiller http status code'
                    metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                    metric.add_metric(labels.values(), response.status_code)
                    metrics[mkey] = metric

                    mkey = '{}_http_response_size_bytes'.format(self._app)
                    mhelp = 'Chiller http response size in bytes'
                    metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                    metric.add_metric(labels.values(), len(response.content))
                    metrics[mkey] = metric

                    mkey = '{}_http_response_duration_seconds'.format(self._app)
                    mhelp = 'Chiller http response elapsed time in seconds'
                    metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                    metric.add_metric(labels.values(), response.elapsed.total_seconds())
                    metrics[mkey] = metric

                    if response.status_code == 200 and len(response.content):
                        xml = xmltodict.parse(response.content)
                        for part in xml.get('PCOWEB', {}).get('PCO', {}):
                            for pair in xml.get('PCOWEB', {}).get('PCO', {}).get(part, {}).get('O', {}):
                                mkey, mhelp = self._metrics.get(part, {}).get(pair['@I'], (None, None))

                                if not mkey:
                                    continue

                                metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                                metric.add_metric(labels.values(), float(pair['@V']))
                                metrics[mkey] = metric

                mkey = '{}_up'.format(self._app)
                mhelp = 'Chiller HTTP up and running'
                metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                metric.add_metric(labels.values(), 1)
                metrics[mkey] = metric

                mkey = '{}_error'.format(self._app)
                mhelp = 'Chiller HTTP up and running'
                metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                metric.add_metric(labels.values(), 0)
                metrics[mkey] = metric

            except Exception as exception:
                mkey = '{}_up'.format(self._app)
                mhelp = 'Chiller HTTP up and running'
                metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                metric.add_metric(labels.values(), 0)
                metrics[mkey] = metric

                labels['error'] = type(exception).__name__
                mkey = '{}_error'.format(self._app)
                mhelp = 'Chiller HTTP up and running'
                metric = metrics.get(mkey, GaugeMetricFamily(mkey, mhelp, labels=labels.keys()))
                metric.add_metric(labels.values(), 1)
                metrics[mkey] = metric

        for metric in metrics.values():
            yield metric
