import argparse
import os

from configparser import ConfigParser, DuplicateOptionError
from urllib.parse import urlsplit, urlunsplit

# default filepath for pcoweb.conf
PCOWEB_CONF = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'pcoweb.conf')
APP_PREFIX = 'chiller'
APP_TIMEOUT = 2.0

# default values for config
DEFAULTS = {
    'scheme': 'http',
    'port': '80',
    'path': '/usr-cgi/xml.cgi',
    'query': 'N%%7CD%%7C1%%7C207%%7CI%%7C1%%7C207%%7CA%%7C1%%7C207',
    'timeout': str(APP_TIMEOUT)
}


def parse_args():
    """cmd argument parser - returns instance of ArgumentParser"""
    parser = argparse.ArgumentParser(
        description='chiller exporter args'
    )
    parser.add_argument(
        'chillers',
        metavar='chillers',
        nargs='*',
        help='rittal chiller urls as "instance=url ..."'
    )
    parser.add_argument(
        '-a', '--app',
        metavar='app',
        required=False,
        help='prometheus application prefix',
        default=APP_PREFIX
    )
    parser.add_argument(
        '-c', '--config',
        metavar='config',
        required=False,
        help='config file',
    )
    parser.add_argument(
        '-m', '--metric-conf',
        metavar='metric',
        required=False,
        help='config file for pcoweb metrics',
        default=PCOWEB_CONF
    )
    parser.add_argument(
        '-p', '--port',
        metavar='port',
        required=False,
        type=int,
        help='Listen to this port',
        default=9118
    )
    parser.add_argument(
        '-t', '--timeout',
        metavar='timeout',
        required=False,
        type=float,
        help='chiller http timeout',
        default=APP_TIMEOUT
    )

    return parser.parse_args()


def parse_config(args):
    """
    parse arguments given on command line and given config file for chiller configuration
    - returns dictionary of chillers { prom_instance_label: (chiller_url,timeout), ...}
    """

    urls = {}

    for arg in args.chillers:
        label, value = arg.split('=', 1)

        if not urlsplit(label).scheme and urlsplit(value).scheme:
            urls[label] = (value, args.timeout)
        elif urlsplit(label).scheme and urlsplit(label).netloc:
            urls[urlsplit(label).netloc] = (label, args.timeout)

    if not args.config:
        return urls

    parser = ConfigParser(defaults=DEFAULTS)

    parser.read(args.config)

    for instance in parser.sections():
        values = {k: v for k, v in parser.items(instance)}
        if 'hostname' not in values:
            continue
        url = urlunsplit((values['scheme'], values['hostname'], values['path'], values['query'], ''))
        urls[instance] = (url, float(values['timeout']))

    return urls


def parse_metrics_config(filename, app, sep='|'):
    """
    load metric configuration from config file
    - returns dictionary of dictionaries { value_group : { value_id: (metric_name, value_help), ... }, ...}
    """

    parser = ConfigParser(interpolation=None)
    parser.read(filename)

    names = {}
    metrics = {}
    for section in parser.sections():
        values = {'{}_{}'.format(app, k): v for k, v in parser.items(section) if k != 'prefix'}
        for k in values:
            if k in names:
                raise DuplicateOptionError("option '{}' already exists in section '{}'".format(k, names[k]), k)

            names[k] = section
            if section not in metrics:
                metrics[section] = {}

            i, mhelp = values[k].split(sep, 1)
            metrics[section][i] = (k, mhelp)

    return metrics
